<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>JSON Data Table</title>
</head>
<body>
    <h1>JSON Data Table</h1>
    <table>
        <thead>
            <tr>
                <th>Key</th>
                <th>Value</th>
            </tr>
        </thead>
        <tbody>
            {% for fullpath, value in data.items() %}
                <tr>
                    <td>{{ fullpath }}</td>
                    <td>{{ value['pre'] }}</td>
                    <td>{{ value['post'] }}</td>
                </tr>
            {% endfor %}
        </tbody>
    </table>
</body>
</html>
